package org.fax4j.spi.http;

import java.util.Properties;
import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.spi.AbstractFax4JClientSpi;
import org.fax4j.util.ReflectionHelper;

/**
 * This class implements the fax client service provider interface.<br>
 * This parial implementation will invoke the requests by sending HTTP requests to a web server that supports
 * sending fax messages.<br>
 * Implementing SPI class will have to implement the createXXXFaxJobHTTPRequest methods.<br>
 * These methods will return the HTTP request to be sent for that fax job operation.
 * In case the method returns null, this class will throw an UnsupportedOperationException exception.
 * <br>
 * The configuration of the fax4j framework is made up of 3 layers.<br>
 * The configuration is based on simple properties.<br>
 * Each layer overrides the lower layers by adding/changing the property values.<br>
 * The first layer is the internal fax4j.properties file located in the fax4j jar.<br>
 * This layer contains the preconfigured values for the fax4j framework and can be changed
 * by updating these properties in the higher layers.<br>
 * The second layer is the external fax4j.properties file that is located on the classpath.<br>
 * This file is optional and provides the ability to override the internal configuration for the
 * entire fax4j framework.<br>
 * The top most layer is the optional java.util.Properties object provided by the external classes
 * when creating a new fax client.<br>
 * These properties enable to override the configuration of the lower 2 layers.<br>
 * <br>
 * <b>SPI Status (Draft, Beta, Stable): </b>Beta<br>
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.client.class.name</td>
 * 		<td>The HTTP client class name used to submit the HTTP requests</td>
 * 		<td>org.fax4j.spi.http.ApacheHTTPClient</td>
 * 		<td>org.fax4j.spi.http.ApacheHTTPClient</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.host.name</td>
 * 		<td>The target (service provider) host name.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>true</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.port</td>
 * 		<td>The target port number.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.ssl</td>
 * 		<td>True to use SSL (HTTPS), false for simple HTTP</td>
 * 		<td>false</td>
 * 		<td>false</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.resource</td>
 * 		<td>The default HTTP resource of the request</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.http.url.parameters</td>
 * 		<td>The default HTTP URL parameters of the request</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * </tr>
 * </table>
 * <br>
 * <b>Limitations:</b><br>
 * <ul>
 * 	<li>This SPI provides only partial implementation (this is an abstract class).
 * </ul>
 * <br>
 * <b>Dependencies:</b><br>
 * <ul>
 * 	<li>HTTP client required jars.
 * </ul>
 * <br>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.09
 * @since	0.1
 */
public abstract class AbstractHTTPFaxClientSpi extends AbstractFax4JClientSpi
{
	/**The HTTP client used to submit the HTTP requests*/
	private HTTPClient httpClient;
	/**The HTTP client configuration*/
	private HTTPClientConfiguration httpClientConfiguration;
	/**The default HTTP resource*/
	private String resource;
	/**The default HTTP URL parameters*/
	private String urlParameters;

	/**
	 * This class holds the SPI configuration constants.
	 * 
	 * @author 	Sagie Gur-Ari
	 * @version 1.03
	 * @since	0.1
	 */
	public enum FaxClientSpiConfigurationConstants
	{
		/**The HTTP client class name property key*/
		HTTP_CLIENT_CLASS_NAME_PROPERTY_KEY("org.fax4j.spi.http.client.class.name"),
		/**The HTTP resource property key*/
		HTTP_RESOURCE_PROPERTY_KEY("org.fax4j.spi.http.resource"),
		/**The HTTP URL parameters property key*/
		HTTP_URL_PARAMETERS_PROPERTY_KEY("org.fax4j.spi.http.url.parameters");

		/**The string value*/
		private String value;

		/**
		 * This is the class constructor.
		 * 
		 * @param	value
		 * 			The string value
		 */
		private FaxClientSpiConfigurationConstants(String value)
		{
			this.value=value;
		}
		
		/**
		 * This function returns the string value.
		 * 
		 * @return	The string value
		 */
		@Override
		public final String toString()
		{
			return this.value;
		}
	}
	
	/**
	 * This is the default constructor.
	 */
	public AbstractHTTPFaxClientSpi()
	{
		super();
	}
	
	/**
	 * This function initializes the fax client SPI.
	 */
	@Override
	protected void initializeImpl()
	{
		String value=this.getConfigurationValue(FaxClientSpiConfigurationConstants.HTTP_CLIENT_CLASS_NAME_PROPERTY_KEY);
		if(value==null)
		{
			value=ApacheHTTPClient.class.getName();
		}

		//create HTTP client
		this.httpClient=(HTTPClient)ReflectionHelper.createInstance(value);
		
		//create configuration
		this.httpClientConfiguration=this.httpClient.createHTTPClientConfiguration(this);
		
		//get resource
		this.resource=this.getConfigurationValue(FaxClientSpiConfigurationConstants.HTTP_RESOURCE_PROPERTY_KEY);
		
		//get URL parameters
		this.urlParameters=this.getConfigurationValue(FaxClientSpiConfigurationConstants.HTTP_URL_PARAMETERS_PROPERTY_KEY);
	}
	
	/**
	 * Returns the HTTP client.
	 *  
	 * @return	The HTTP client
	 */
	protected final HTTPClient getHTTPClient()
	{
		return this.httpClient;
	}

	/**
	 * Returns the HTTP client configuration.
	 *  
	 * @return	The HTTP client configuration
	 */
	protected final HTTPClientConfiguration getHTTPClientConfiguration()
	{
		return this.httpClientConfiguration;
	}
	
	/**
	 * This function returns the HTTP resource.
	 * 
	 * @return	The HTTP resource
	 */
	protected final String getHTTPResource()
	{
		return this.resource;
	}
	
	/**
	 * This function returns the HTTP URL parameters.
	 * 
	 * @return	The HTTP URL parameters
	 */
	protected final String getHTTPURLParameters()
	{
		return this.urlParameters;
	}
	
	/**
	 * This function will submit a new fax job.<br>
	 * The fax job ID may be populated by this method in the provided
	 * fax job object.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void submitFaxJobImpl(FaxJob faxJob)
	{
		//create HTTP request
		HTTPRequest httpRequest=this.createSubmitFaxJobHTTPRequest(faxJob);
		
		//submit HTTP request
		this.submitHTTPRequest(faxJob,httpRequest);
	}
	
	/**
	 * This function will suspend an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void suspendFaxJobImpl(FaxJob faxJob)
	{
		//create HTTP request
		HTTPRequest httpRequest=this.createSuspendFaxJobHTTPRequest(faxJob);
		
		//submit HTTP request
		this.submitHTTPRequest(faxJob,httpRequest);
	}
	
	/**
	 * This function will resume an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void resumeFaxJobImpl(FaxJob faxJob)
	{
		//create HTTP request
		HTTPRequest httpRequest=this.createResumeFaxJobHTTPRequest(faxJob);
		
		//submit HTTP request
		this.submitHTTPRequest(faxJob,httpRequest);
	}
	
	/**
	 * This function will cancel an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void cancelFaxJobImpl(FaxJob faxJob)
	{
		//create HTTP request
		HTTPRequest httpRequest=this.createCancelFaxJobHTTPRequest(faxJob);
		
		//submit HTTP request
		this.submitHTTPRequest(faxJob,httpRequest);
	}
	
	/**
	 * Submits the HTTP request and returns the HTTP response.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @param 	httpRequest
	 * 			The HTTP request to send
	 */
	protected void submitHTTPRequest(FaxJob faxJob,HTTPRequest httpRequest)
	{
		if(httpRequest==null)
		{
			this.throwUnsupportedException();
		}
		else
		{
			//setup default header properties
			Properties headerProperties=httpRequest.getHeaderProperties();
			if(headerProperties==null)
			{
				headerProperties=new Properties();
				httpRequest.setHeaderProperties(headerProperties);
			}
			
			//setup resource
			if(httpRequest.getResource()==null)
			{
				httpRequest.setResource(this.resource);
			}
	
			//setup URL parameters
			if(httpRequest.getParametersText()==null)
			{
				httpRequest.setParametersText(this.urlParameters);
			}
			
			//submit HTTP request
			HTTPResponse httpResponse=this.submitHTTPRequestImpl(faxJob,httpRequest);
	
			//validate response status code
			int statusCode=httpResponse.getStatusCode();
			if(statusCode>=400)
			{
				throw new FaxException("Error while invoking HTTP request, return status code: "+statusCode);
			}
			
			//update fax job
			this.updateFaxJob(faxJob,httpResponse);
		}
	}
	
	/**
	 * Submits the HTTP request and returns the HTTP response.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @param 	httpRequest
	 * 			The HTTP request to send
	 * @return	The HTTP response
	 */
	protected HTTPResponse submitHTTPRequestImpl(FaxJob faxJob,HTTPRequest httpRequest)
	{
		return this.httpClient.submitHTTPRequest(this,faxJob,httpRequest,this.httpClientConfiguration);
	}
	
	/**
	 * Updates the fax job based on the data from the HTTP response.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @param 	httpResponse
	 * 			The HTTP response
	 */
	protected abstract void updateFaxJob(FaxJob faxJob,HTTPResponse httpResponse);
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	protected abstract HTTPRequest createSubmitFaxJobHTTPRequest(FaxJob faxJob);
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	protected abstract HTTPRequest createSuspendFaxJobHTTPRequest(FaxJob faxJob);
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	protected abstract HTTPRequest createResumeFaxJobHTTPRequest(FaxJob faxJob);
	
	/**
	 * Creates the HTTP request from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The HTTP request to send
	 */
	protected abstract HTTPRequest createCancelFaxJobHTTPRequest(FaxJob faxJob);
}